clear
set more off
set matsize 800
global date "`c(current_date)'"

*ssc install egenmore

*************************************************
*DEFINE PATHS:

global DRIVE "."  //<--- Adjust!

global data "${DRIVE}/Data"
global dtafiles "${DRIVE}/Dta Files"
sysdir set PLUS "${DRIVE}/ado"
**************************************************




********************************************************************************
********************************************************************************
*** PREPARE DATASETS
********************************************************************************
********************************************************************************

*** Subsidies, Grants, Social Contributions
import excel "$data\Subsidies (% Expenses).xlsx", clear firstrow

reshape long y, i( CountryName CountryCode IndicatorName IndicatorCode ) j(year)

rename y SubsidiesPercExpense

save "$dtafiles\Subsidies Percent Expense.dta", replace



*** Subsidies, Grants, Social Contributions
import excel "$data\Expense (% of GDP).xls", clear firstrow

reshape long y, i( CountryName CountryCode IndicatorName IndicatorCode ) j(year)

rename y ExpensePercGDP

save "$dtafiles\Expense Percent GDP.dta", replace



*** Tax revenue (from IMF WoRLD)
use "$data\IMF_WoRLD STATA File 1990-2021 (wide).dta", clear

rename cname Country
tostring ccode,gen(WEOCountryCode)

drop *_src 

gen taxshare = tax/100 
lab var taxshare "Tax revenue as a share of GDP"

keep Country WEOCountryCode year tax taxshare

save "$dtafiles\Tax revenue perc GDP.dta", replace



*** Government Revenue
import excel "$data\WEOOct2023all.xlsx", clear firstrow

keep if WEOSubjectCode == "GGR_NGDP"

reshape long y, i( WEOCountryCode ISO WEOSubjectCode Country SubjectDescriptor SubjectNotes Units Scale CountrySeriesspecificNotes EstimatesStartAfter ) j(year)

rename y revenue

//transform revenuenm to numerical 
replace revenue="" if revenue=="n/a" 
destring revenue, gen(revenuenm)

//make expenditurenm in share and not in % of gdp: 
generate revenueshare =revenuenm/100

label var revenueshare "General Government Revenue, share of GDP (IMF WEO October 2023)"

save "$dtafiles\General Government Revenue.dta", replace



*** Government Expenditure
import excel "$data\WEOOct2023all.xlsx", clear firstrow

keep if WEOSubjectCode == "GGX_NGDP"

reshape long y, i( WEOCountryCode ISO WEOSubjectCode Country SubjectDescriptor SubjectNotes Units Scale CountrySeriesspecificNotes EstimatesStartAfter ) j(year)

rename y expenditure

//transform revenuenm to numerical 
replace expenditure="" if expenditure=="n/a" 
destring expenditure, gen(expenditurenum)

//make expenditurenm in share and not in % of gdp: 
generate expshare =expenditurenum/100

label var expshare "General government total expenditure, share of GDP (IMF WEO October 2023)"

save "$dtafiles\General Government Total Expenditure.dta", replace



*** Real GDP LCU
import excel "$data\Real GDP LCU.xls", clear sheet("Data") firstrow

reshape long y, i( CountryName CountryCode IndicatorName IndicatorCode ) j(year)

rename y GDP

save "$dtafiles\Real GDP LCU_M.dta", replace



*** GDP Per Capita USD (to order countries according to incoem level)
import excel "$data\GDP per capita (constant 2015 USD).xls", clear sheet("Data") firstrow

reshape long y, i( CountryName CountryCode IndicatorName IndicatorCode ) j(year)

rename y gdppcusd

gen gdppcusd2000x = gdppcusd if year==2000

bysort CountryName : egen gdppcusd2000 = max(gdppcusd2000x)
drop gdppcusd2000x

save "$dtafiles\Real GDPPC USD.dta", replace



*** WEO to WDI key
import excel "$data\WEO to WDI Codes.xlsx", firstrow clear sheet("Final")

save "$dtafiles\WEO to WDI Codes.dta", replace



*** Merge all datasets
use "$dtafiles\General Government Revenue.dta", clear

merge 1:1 ISO year using "$dtafiles\General Government Total Expenditure.dta"
drop _m

merge m:1  ISO using "$dtafiles\WEO to WDI Codes.dta", keepusing("ISO" "CountryCode")
drop _m

*Merge WDI Dataset: Real GDP LCU
merge 1:1 CountryCode year using "$dtafiles\Real GDP LCU_M.dta"
keep if _m==1 | _m==3
drop _m

*Merge WDI Dataset: Real GDP per Capita USD
merge 1:1 CountryCode year using "$dtafiles\Real GDPPC USD.dta"
keep if _m==1 | _m==3
drop _m	

merge 1:1 CountryCode year using "$dtafiles\Subsidies Percent Expense.dta"
keep if _m==1 | _m==3
drop _m	

merge 1:1 CountryCode year using "$dtafiles\Expense Percent GDP.dta"
keep if _m==1 | _m==3
drop _m	

merge 1:1 WEOCountryCode year using "$dtafiles\Tax revenue perc GDP.dta"
keep if _m==1 | _m==3
drop _m	

keep if inrange(year,1999,2022)

tab Country if CountryName==""
drop if CountryName==""






********************************************************************************
********************************************************************************
*** COMPUTE CYCLICAL COMPONENTS (HP filter)
********************************************************************************
********************************************************************************

////////////////////////////////////////////////////////////////////
*Remove if GDP or Revenue not both present: 
drop if   GDP==.  | revenueshare==. | expshare==. 
////////////////////////////////////////////////////////////////////

ren CountryCode wbcode
ren ISO imfcode


*Generate MENA Dummy
generate MENA=.
replace MENA=1 if wbcode=="DZA" | wbcode=="BHR" | wbcode=="DJI" | wbcode=="EGY" | wbcode=="IRN" | wbcode=="IRQ" | wbcode=="JOR" | wbcode=="KWT"  |wbcode=="LBN" | wbcode=="LBY" | wbcode=="MAR" | wbcode=="OMN" | wbcode=="PSE" | wbcode=="QAT" | wbcode=="SAU" | wbcode=="SYR" | wbcode=="TUN" | wbcode=="ARE" | wbcode=="YEM" 


*Generate Egypt Comparators Dummy
gen comparator=.
replace comparator=1 if inlist(Country, "Indonesia", "Jordan", "Morocco", "Peru", "Philippines", "Poland") |  inlist(Country, "Türkiye", "Ukraine", "Vietnam", "Mexico", "Pakistan", "Malaysia", "Sri Lanka", "Tunisia")



*Declara Panel Dataset
encode wbcode, gen (wbcodenm)
tsset wbcodenm year
sort wbcodenm year



*1)HP filter for: log GDP LCU
gen ln_gdp = ln(GDP)
label var ln_gdp "Log of Real GDP, LCU (WDI)"
label var GDP "Real GDP, LCU (WDI)"
 
*create HP filter with smoothing lambda = 6.25
tsfilter hp gdp_hp = ln_gdp, smooth(6.25)
label var gdp_hp "HP TimeSeries Cyclical Component Filter on Log of Real GDP LCU, smoothing Lambda 6.25"



*2) HP Filter for: log Real Revenue LCU
* Real Revenue LCU = Revenue Share of GDP * Real GDP LCU
gen real_rev = revenueshare*GDP
label var real_rev "Real Revenue LCU (Calculated)"

* Log of Real Revenue LCU
gen log_real_rev = ln(real_rev)
label var log_real_rev "Log of Real Revenue LCU (Calculated)"

* Apply HP Filter with 6.25 smoothing lambda: 
tsfilter hp rev_hp = log_real_rev, smooth(6.25)
label var rev_hp "HP TimeSeries Cyclical Component Filter on Log of Real Revenue LCU, smoothing Lambda 6.25"



*3) HP Filter for: Revenue SHARE
tsfilter hp revshare_hp = revenueshare, smooth(6.25)



*4) HP Filter for: log Real Expenditure LCU
* Real Expenditure LCU = Expenditure Share of GDP * Real GDP LCU
gen real_exp = expshare*GDP
label var real_exp "Real Expenditure LCU (Calculated)"

* Log of Real Expenditure LCU
gen log_realexp = ln(real_exp)
label var log_realexp "Log of Real Expenditure LCU (Calculated)"

* Apply HP Filter with 6.25 smoothing lambda: 
tsfilter hp exp_hp = log_realexp, smooth(6.25)
label var exp_hp "HP TimeSeries Cyclical Component Filter on Log of Real Expenditure LCU, smoothing Lambda 6.25"



*5) HP Filter for: Expenditure SHARE
tsfilter hp expshare_hp = expshare, smooth(6.25)



*6) General government (structural) balance:
gen balance_hp = rev_hp -  exp_hp



* HP Filter for: log Real Tax revenue LCU (MENA only)
gen real_tax = taxshare*GDP
label var real_tax "Real Tax Revenue LCU (Calculated)"

gen log_real_tax = ln(real_tax)
label var log_real_tax "Log of Real Tax Revenue LCU (Calculated)"

preserve
keep if MENA==1
tsfilter hp tax_hp = log_real_tax  , smooth(6.25)
label var tax_hp "HP TimeSeries Cyclical Component Filter on Log of Real Tax Revenue LCU, smoothing Lambda 6.25"
keep wbcode year tax_hp
save "$dtafiles\temp_taxHP.dta", replace
restore

merge 1:1 wbcode year using "$dtafiles\temp_taxHP.dta"
drop _m



* HP Filter for: Subsidies (MENA only)
gen subshareGDP = SubsidiesPercExpense * ExpensePercGDP /10000
*amend values for Egypt with internal data (see footnote 2 in the paper)
replace subshareGDP =. if Country=="Egypt"
replace subshareGDP = 0.049222 if Country=="Egypt" & year==2003
replace subshareGDP = 0.051004 if Country=="Egypt" & year==2004
replace subshareGDP = 0.055164 if Country=="Egypt" & year==2005
replace subshareGDP = 0.111538 if Country=="Egypt" & year==2006
replace subshareGDP = 0.078467 if Country=="Egypt" & year==2007
replace subshareGDP = 0.103150 if Country=="Egypt" & year==2008
replace subshareGDP = 0.121889 if Country=="Egypt" & year==2009
replace subshareGDP = 0.085342 if Country=="Egypt" & year==2010
replace subshareGDP = 0.089800 if Country=="Egypt" & year==2011
replace subshareGDP = 0.089684 if Country=="Egypt" & year==2012
replace subshareGDP = 0.105941 if Country=="Egypt" & year==2013
replace subshareGDP = 0.107314 if Country=="Egypt" & year==2014
replace subshareGDP = 0.081251 if Country=="Egypt" & year==2015
replace subshareGDP = 0.074195 if Country=="Egypt" & year==2016
replace subshareGDP = 0.075691 if Country=="Egypt" & year==2017
replace subshareGDP = 0.070588 if Country=="Egypt" & year==2018
replace subshareGDP = 0.051369 if Country=="Egypt" & year==2019
replace subshareGDP = 0.037255 if Country=="Egypt" & year==2020
replace subshareGDP = 0.039604 if Country=="Egypt" & year==2021
replace subshareGDP = 0.043788 if Country=="Egypt" & year==2022
replace subshareGDP = 0.043596 if Country=="Egypt" & year==2023

gen real_sub = subshareGDP*GDP
label var real_sub "Real Subsidies LCU (Calculated)"

* Log of Real Subsidies LCU
gen log_realsub = ln(real_sub)
label var log_realsub "Log of Real Subsidies LCU (Calculated)"

* Apply HP Filter with 6.25 smoothing lambda: 
preserve
*need to exclude gaps in time series
drop if Country=="Tunisia" & year==1999
drop if Country=="United Arab Emirates" & year==1999
drop if Country=="Morocco" & year==1999
keep if log_realsub!=. & MENA==1
tsfilter hp sub_hp = log_realsub, smooth(6.25)
label var sub_hp "HP TimeSeries Cyclical Component Filter on Log of Real Subsidies LCU, smoothing Lambda 6.25"
keep wbcode year sub_hp
save "$dtafiles\temp_subHP.dta", replace
restore


merge 1:1 wbcode year using "$dtafiles\temp_subHP.dta"
drop _m

sort wbcodenm

save "$dtafiles\DATA_rev.dta", replace




********************************************************************************
********************************************************************************
***Correlation Analysis
********************************************************************************
********************************************************************************

foreach var in rev  revshare  exp  balance   tax   sub  {

use "$dtafiles\DATA_rev.dta", clear

*2000-2022
preserve
keep if inrange(year,2000,2022)
egen corrhp_`var'_gdp_00_22 = corr(gdp_hp `var'_hp) , by(wbcodenm)
bysort wbcode: gen count_obs_00_22=_N 
save "$dtafiles\corrhp_`var'_gdp_00_22.dta", replace
restore

*2000-2011
preserve
keep if inrange(year,2000,2011)
egen corrhp_`var'_gdp_00_11 = corr(gdp_hp `var'_hp) , by(wbcodenm)
bysort wbcode: gen count_obs_00_11=_N 
save "$dtafiles\corrhp_`var'_gdp_00_11.dta", replace
restore

*2012-2022
preserve
keep if inrange(year,2012,2022)
egen corrhp_`var'_gdp_12_22 = corr(gdp_hp `var'_hp) , by(wbcodenm)
bysort wbcode: gen count_obs_12_22=_N 
save "$dtafiles\corrhp_`var'_gdp_12_22.dta", replace
restore


merge 1:1 wbcode year using "$dtafiles\corrhp_`var'_gdp_00_22.dta"
keep if _m ==3 //drop year 1999
drop _m

merge 1:1 wbcode year using "$dtafiles\corrhp_`var'_gdp_00_11.dta"
drop _m

merge 1:1 wbcode year using "$dtafiles\corrhp_`var'_gdp_12_22.dta"
drop _m


*collapsing to 1 obs per country
collapse (first)  Country MENA comparator   (max) corrhp_`var'_gdp_00_22 count_obs_00_22     corrhp_`var'_gdp_00_11  count_obs_00_11    corrhp_`var'_gdp_12_22  count_obs_12_22  gdppcusd2000 , by(wbcode)


drop if corrhp_`var'_gdp_00_22 ==. | corrhp_`var'_gdp_00_11==.  |  corrhp_`var'_gdp_12_22==.
drop if gdppcusd2000 ==.


gsort MENA -gdppcusd2000
export excel "Cyclicality Results - FIGURES.xlsx", firstrow(variables)  sheet("`var'", replace)

}



/////////////
ex
/////////////




